@@ -16,8 +16,8 @@
 
    You should have received a copy of the GNU Library General Public
    License along with the Mate Library; see the file COPYING.LIB.  If not,
-   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 
    Authors: Jan Arne Petersen <jpetersen@uni-bonn.de>
 */
@@ -33,9 +33,7 @@
 
 #include <glib/gi18n.h>
 
-#include <eel/eel-mateconf-extensions.h>
 #include <eel/eel-glib-extensions.h>
-#include <eel/eel-preferences.h>
 
 #include <libcaja-private/caja-column-chooser.h>
 #include <libcaja-private/caja-column-utilities.h>
@@ -76,9 +74,9 @@
 
 static const char * const default_view_values[] =
 {
-    "icon_view",
-    "list_view",
-    "compact_view",
+    "icon-view",
+    "list-view",
+    "compact-view",
     NULL
 };
 
@@ -96,11 +94,15 @@ static const char * const zoom_values[] =
 
 static const char * const sort_order_values[] =
 {
+    "manually",
     "name",
+    "directory",
     "size",
     "type",
-    "modification_date",
+    "mtime",
+    "atime",
     "emblems",
+    "trash-time",
     NULL
 };
 
@@ -115,7 +117,7 @@ static const char * const date_format_values[] =
 static const char * const preview_values[] =
 {
     "always",
-    "local_only",
+    "local-only",
     "never",
     NULL
 };
@@ -150,7 +152,7 @@ static const char * const executable_text_values[] =
     NULL
 };
 
-static const guint thumbnail_limit_values[] =
+static const guint64 thumbnail_limit_values[] =
 {
     102400,
     512000,
@@ -172,6 +174,8 @@ static const char * const icon_captions_components[] =
     NULL
 };
 
+static void caja_file_management_properties_dialog_update_media_sensitivity (GtkBuilder *builder);
+
 static void
 caja_file_management_properties_size_group_create (GtkBuilder *builder,
         char *prefix,
@@ -258,16 +262,18 @@ caja_file_management_properties_dialog_response_cb (GtkDialog *parent,
             break;
         case 4:
             section = "goscaja-60";
+            break;
+        case 5:
+            section = "goscaja-61";
+            break;
         }
         preferences_show_help (GTK_WINDOW (parent), "user-guide", section);
     }
     else if (response_id == GTK_RESPONSE_CLOSE)
     {
-        /* remove mateconf monitors */
-        eel_mateconf_monitor_remove ("/apps/caja/icon_view");
-        eel_mateconf_monitor_remove ("/apps/caja/list_view");
-        eel_mateconf_monitor_remove ("/apps/caja/preferences");
-        eel_mateconf_monitor_remove ("/desktop/mate/file_views");
+        g_signal_handlers_disconnect_by_func (caja_media_preferences,
+                                              caja_file_management_properties_dialog_update_media_sensitivity,
+                                              builder);
     }
 }
 
@@ -282,8 +288,12 @@ columns_changed_callback (CajaColumnChooser *chooser,
                                       &visible_columns,
                                       &column_order);
 
-    eel_preferences_set_string_array (CAJA_PREFERENCES_LIST_VIEW_DEFAULT_VISIBLE_COLUMNS, visible_columns);
-    eel_preferences_set_string_array (CAJA_PREFERENCES_LIST_VIEW_DEFAULT_COLUMN_ORDER, column_order);
+    g_settings_set_strv (caja_list_view_preferences,
+                         CAJA_PREFERENCES_LIST_VIEW_DEFAULT_VISIBLE_COLUMNS,
+                         (const char * const *)visible_columns);
+    g_settings_set_strv (caja_list_view_preferences,
+                         CAJA_PREFERENCES_LIST_VIEW_DEFAULT_COLUMN_ORDER,
+                         (const char * const *)column_order);
 
     g_strfreev (visible_columns);
     g_strfreev (column_order);
@@ -297,7 +307,7 @@ free_column_names_array (GPtrArray *column_names)
 }
 
 static void
-create_icon_caption_combo_box_items (GtkComboBox *combo_box,
+create_icon_caption_combo_box_items (GtkComboBoxText *combo_box,
                                      GList *columns)
 {
     GList *l;
@@ -306,7 +316,7 @@ create_icon_caption_combo_box_items (GtkComboBox *combo_box,
     column_names = g_ptr_array_new ();
 
     /* Translators: this is referred to captions under icons. */
-    gtk_combo_box_append_text (combo_box, _("None"));
+    gtk_combo_box_text_append_text (combo_box, _("None"));
     g_ptr_array_add (column_names, g_strdup ("none"));
 
     for (l = columns; l != NULL; l = l->next)
@@ -329,7 +339,7 @@ create_icon_caption_combo_box_items (GtkComboBox *combo_box,
             continue;
         }
 
-        gtk_combo_box_append_text (combo_box, label);
+        gtk_combo_box_text_append_text (combo_box, label);
         g_ptr_array_add (column_names, name);
 
         g_free (label);
@@ -370,8 +380,9 @@ icon_captions_changed_callback (GtkComboBox *combo_box,
     }
     g_ptr_array_add (captions, NULL);
 
-    eel_preferences_set_string_array (CAJA_PREFERENCES_ICON_VIEW_CAPTIONS,
-                                      (char **)captions->pdata);
+    g_settings_set_strv (caja_icon_view_preferences,
+                         CAJA_PREFERENCES_ICON_VIEW_CAPTIONS,
+                         (const char **)captions->pdata);
     g_ptr_array_free (captions, TRUE);
 }
 
@@ -410,12 +421,12 @@ update_caption_combo_box (GtkBuilder *builder,
 }
 
 static void
-update_icon_captions_from_mateconf (GtkBuilder *builder)
+update_icon_captions_from_settings (GtkBuilder *builder)
 {
     char **captions;
     int i, j;
 
-    captions = eel_preferences_get_string_array (CAJA_PREFERENCES_ICON_VIEW_CAPTIONS);
+    captions = g_settings_get_strv (caja_icon_view_preferences, CAJA_PREFERENCES_ICON_VIEW_CAPTIONS);
     if (captions == NULL)
         return;
 
@@ -450,7 +461,7 @@ caja_file_management_properties_dialog_setup_icon_caption_page (GtkBuilder *buil
     int i;
     gboolean writable;
 
-    writable = eel_preferences_key_is_writable (CAJA_PREFERENCES_ICON_VIEW_CAPTIONS);
+    writable = g_settings_is_writable (caja_icon_view_preferences, CAJA_PREFERENCES_ICON_VIEW_CAPTIONS);
 
     columns = caja_get_common_columns ();
 
@@ -461,7 +472,7 @@ caja_file_management_properties_dialog_setup_icon_caption_page (GtkBuilder *buil
         combo_box = GTK_WIDGET (gtk_builder_get_object (builder,
                                 icon_captions_components[i]));
 
-        create_icon_caption_combo_box_items (GTK_COMBO_BOX (combo_box), columns);
+        create_icon_caption_combo_box_items (GTK_COMBO_BOX_TEXT (combo_box), columns);
         gtk_widget_set_sensitive (combo_box, writable);
 
         g_signal_connect (combo_box, "changed",
@@ -471,44 +482,47 @@ caja_file_management_properties_dialog_setup_icon_caption_page (GtkBuilder *buil
 
     caja_column_list_free (columns);
 
-    update_icon_captions_from_mateconf (builder);
+    update_icon_captions_from_settings (builder);
 }
 
 static void
 create_date_format_menu (GtkBuilder *builder)
 {
-    GtkWidget *combo_box;
+    GtkComboBoxText *combo_box;
     gchar *date_string;
     time_t now_raw;
     struct tm* now;
 
-    combo_box = GTK_WIDGET (gtk_builder_get_object (builder,
-                            CAJA_FILE_MANAGEMENT_PROPERTIES_DATE_FORMAT_WIDGET));
+    combo_box = GTK_COMBO_BOX_TEXT
+            (gtk_builder_get_object (builder,
+                                     CAJA_FILE_MANAGEMENT_PROPERTIES_DATE_FORMAT_WIDGET));
 
     now_raw = time (NULL);
     now = localtime (&now_raw);
 
     date_string = eel_strdup_strftime ("%c", now);
-    gtk_combo_box_append_text (GTK_COMBO_BOX (combo_box), date_string);
+    gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo_box), date_string);
     g_free (date_string);
 
     date_string = eel_strdup_strftime ("%Y-%m-%d %H:%M:%S", now);
-    gtk_combo_box_append_text (GTK_COMBO_BOX (combo_box), date_string);
+    gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo_box), date_string);
     g_free (date_string);
 
     date_string = eel_strdup_strftime (_("today at %-I:%M:%S %p"), now);
-    gtk_combo_box_append_text (GTK_COMBO_BOX (combo_box), date_string);
+    gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo_box), date_string);
     g_free (date_string);
 }
 
 static void
-set_columns_from_mateconf (CajaColumnChooser *chooser)
+set_columns_from_settings (CajaColumnChooser *chooser)
 {
     char **visible_columns;
     char **column_order;
 
-    visible_columns = eel_preferences_get_string_array (CAJA_PREFERENCES_LIST_VIEW_DEFAULT_VISIBLE_COLUMNS);
-    column_order = eel_preferences_get_string_array (CAJA_PREFERENCES_LIST_VIEW_DEFAULT_COLUMN_ORDER);
+    visible_columns = g_settings_get_strv (caja_list_view_preferences,
+                                           CAJA_PREFERENCES_LIST_VIEW_DEFAULT_VISIBLE_COLUMNS);
+    column_order = g_settings_get_strv (caja_list_view_preferences,
+                                        CAJA_PREFERENCES_LIST_VIEW_DEFAULT_COLUMN_ORDER);
 
     caja_column_chooser_set_settings (CAJA_COLUMN_CHOOSER (chooser),
                                       visible_columns,
@@ -522,9 +536,11 @@ static void
 use_default_callback (CajaColumnChooser *chooser,
                       gpointer user_data)
 {
-    eel_preferences_unset (CAJA_PREFERENCES_LIST_VIEW_DEFAULT_VISIBLE_COLUMNS);
-    eel_preferences_unset (CAJA_PREFERENCES_LIST_VIEW_DEFAULT_COLUMN_ORDER);
-    set_columns_from_mateconf (chooser);
+    g_settings_reset (caja_list_view_preferences,
+                      CAJA_PREFERENCES_LIST_VIEW_DEFAULT_VISIBLE_COLUMNS);
+    g_settings_reset (caja_list_view_preferences,
+                      CAJA_PREFERENCES_LIST_VIEW_DEFAULT_COLUMN_ORDER);
+    set_columns_from_settings (chooser);
 }
 
 static void
@@ -539,7 +555,7 @@ caja_file_management_properties_dialog_setup_list_column_page (GtkBuilder *build
     g_signal_connect (chooser, "use_default",
                       G_CALLBACK (use_default_callback), chooser);
 
-    set_columns_from_mateconf (CAJA_COLUMN_CHOOSER (chooser));
+    set_columns_from_settings (CAJA_COLUMN_CHOOSER (chooser));
 
     gtk_widget_show (chooser);
     box = GTK_WIDGET (gtk_builder_get_object (builder, "list_columns_vbox"));
@@ -551,7 +567,7 @@ static void
 caja_file_management_properties_dialog_update_media_sensitivity (GtkBuilder *builder)
 {
     gtk_widget_set_sensitive (GTK_WIDGET (gtk_builder_get_object (builder, "media_handling_vbox")),
-                              ! eel_preferences_get_boolean (CAJA_PREFERENCES_MEDIA_AUTORUN_NEVER));
+                              ! g_settings_get_boolean (caja_media_preferences, CAJA_PREFERENCES_MEDIA_AUTORUN_NEVER));
 }
 
 static void
@@ -699,23 +715,212 @@ skip:
     caja_file_management_properties_dialog_update_media_sensitivity (builder);
 }
 
+static void
+bind_builder_bool (GtkBuilder *builder,
+                   GSettings *settings,
+                   const char *widget_name,
+                   const char *prefs)
+{
+    g_settings_bind (settings, prefs,
+                     gtk_builder_get_object (builder, widget_name),
+                     "active", G_SETTINGS_BIND_DEFAULT);
+}
+
+static void
+bind_builder_bool_inverted (GtkBuilder *builder,
+                            GSettings *settings,
+                            const char *widget_name,
+                            const char *prefs)
+{
+    g_settings_bind (settings, prefs,
+                      gtk_builder_get_object (builder, widget_name),
+                      "active", G_SETTINGS_BIND_INVERT_BOOLEAN);
+}
+
+static gboolean
+enum_get_mapping (GValue             *value,
+                  GVariant           *variant,
+                  gpointer            user_data)
+{
+    const char **enum_values = user_data;
+    const char *str;
+    int i;
+
+    str = g_variant_get_string (variant, NULL);
+    for (i = 0; enum_values[i] != NULL; i++) {
+        if (strcmp (enum_values[i], str) == 0) {
+            g_value_set_int (value, i);
+            return TRUE;
+        }
+    }
+
+    return FALSE;
+}
+
+static GVariant *
+enum_set_mapping (const GValue       *value,
+                  const GVariantType *expected_type,
+                  gpointer            user_data)
+{
+    const char **enum_values = user_data;
+
+    return g_variant_new_string (enum_values[g_value_get_int (value)]);
+}
+
+static void
+bind_builder_enum (GtkBuilder *builder,
+                   GSettings *settings,
+                   const char *widget_name,
+                   const char *prefs,
+                   const char **enum_values)
+{
+    g_settings_bind_with_mapping (settings, prefs,
+                                  gtk_builder_get_object (builder, widget_name),
+                                  "active", G_SETTINGS_BIND_DEFAULT,
+                                  enum_get_mapping,
+                                  enum_set_mapping,
+                                  enum_values, NULL);
+}
+
+typedef struct {
+    const guint64 *values;
+    int n_values;
+} UIntEnumBinding;
+
+static gboolean
+uint_enum_get_mapping (GValue             *value,
+               GVariant           *variant,
+               gpointer            user_data)
+{
+    UIntEnumBinding *binding = user_data;
+    guint64 v;
+    int i;
+
+    v = g_variant_get_uint64 (variant);
+    for (i = 0; i < binding->n_values; i++) {
+        if (binding->values[i] >= v) {
+            g_value_set_int (value, i);
+            return TRUE;
+        }
+    }
+
+    return FALSE;
+}
+
+static GVariant *
+uint_enum_set_mapping (const GValue       *value,
+               const GVariantType *expected_type,
+               gpointer            user_data)
+{
+    UIntEnumBinding *binding = user_data;
+
+    return g_variant_new_uint64 (binding->values[g_value_get_int (value)]);
+}
+
+static void
+bind_builder_uint_enum (GtkBuilder *builder,
+            GSettings *settings,
+            const char *widget_name,
+            const char *prefs,
+            const guint64 *values,
+            int n_values)
+{
+    UIntEnumBinding *binding;
+
+    binding = g_new (UIntEnumBinding, 1);
+    binding->values = values;
+    binding->n_values = n_values;
+
+    g_settings_bind_with_mapping (settings, prefs,
+                      gtk_builder_get_object (builder, widget_name),
+                      "active", G_SETTINGS_BIND_DEFAULT,
+                      uint_enum_get_mapping,
+                      uint_enum_set_mapping,
+                      binding, g_free);
+}
+
+typedef struct {
+    GtkWidget *button;
+    const char *value;
+    const char *key;
+    GSettings *settings;
+} RadioBinding;
+
+static void
+radio_binding_setting_changed (GSettings   *settings,
+                   const gchar *key,
+                   gpointer     user_data)
+{
+    RadioBinding *binding = user_data;
+    char *value;
+
+    value = g_settings_get_string (settings, key);
+
+    if (strcmp (value, binding->value) == 0) {
+        /* This will unset the currently active, no need
+           to do that manually */
+        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (binding->button), TRUE);
+    }
+    g_free (value);
+}
+
+static void
+radio_binding_button_toggled (GtkToggleButton *toggle_button,
+                  RadioBinding *binding)
+{
+    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (binding->button))) {
+        g_settings_set_string (binding->settings, binding->key, binding->value);
+    }
+}
+
+static void
+free_radio_binding (gpointer     data,
+            GClosure    *closure)
+{
+    RadioBinding *binding = data;
+
+    g_object_unref (binding->settings);
+    g_free (binding);
+}
+
+static void
+bind_builder_radio (GtkBuilder *builder,
+            GSettings *settings,
+            const char **widget_names,
+            const char *prefs,
+            const char **values)
+{
+    GtkWidget *button;
+    int i;
+    char *detailed_signal;
+    RadioBinding *binding;
+
+    detailed_signal = g_strdup_printf ("changed::%s", prefs);
+
+    for (i = 0; widget_names[i] != NULL; i++) {
+        button = GTK_WIDGET (gtk_builder_get_object (builder, widget_names[i]));
+
+        binding = g_new (RadioBinding, 1);
+        binding->button = button;
+        binding->value = values[i];
+        binding->key = prefs;
+        binding->settings = g_object_ref (settings);
+
+        g_signal_connect (settings, detailed_signal,
+                  G_CALLBACK(radio_binding_setting_changed),
+                  binding);
+
+        g_signal_connect_data (G_OBJECT (button), "toggled",
+                       G_CALLBACK (radio_binding_button_toggled),
+                       binding, free_radio_binding, 0);
+    }
+}
+
 static  void
 caja_file_management_properties_dialog_setup (GtkBuilder *builder, GtkWindow *window)
 {
     GtkWidget *dialog;
 
-    /* setup mateconf stuff */
-    eel_mateconf_monitor_add ("/apps/caja/icon_view");
-    eel_mateconf_preload_cache ("/apps/caja/icon_view", MATECONF_CLIENT_PRELOAD_ONELEVEL);
-    eel_mateconf_monitor_add ("/apps/caja/compact_view");
-    eel_mateconf_preload_cache ("/apps/caja/compact_view", MATECONF_CLIENT_PRELOAD_ONELEVEL);
-    eel_mateconf_monitor_add ("/apps/caja/list_view");
-    eel_mateconf_preload_cache ("/apps/caja/list_view", MATECONF_CLIENT_PRELOAD_ONELEVEL);
-    eel_mateconf_monitor_add ("/apps/caja/preferences");
-    eel_mateconf_preload_cache ("/apps/caja/preferences", MATECONF_CLIENT_PRELOAD_ONELEVEL);
-    eel_mateconf_monitor_add ("/desktop/mate/file_views");
-    eel_mateconf_preload_cache ("/desktop/mate/file_views", MATECONF_CLIENT_PRELOAD_ONELEVEL);
-
     /* setup UI */
     caja_file_management_properties_size_group_create (builder,
             "views_label",
@@ -729,111 +934,107 @@ caja_file_management_properties_dialog_setup (GtkBuilder *builder, GtkWindow *wi
     create_date_format_menu (builder);
 
     /* setup preferences */
-    eel_preferences_builder_connect_bool (builder,
-                                          CAJA_FILE_MANAGEMENT_PROPERTIES_COMPACT_LAYOUT_WIDGET,
-                                          CAJA_PREFERENCES_ICON_VIEW_DEFAULT_USE_TIGHTER_LAYOUT);
-    eel_preferences_builder_connect_bool (builder,
-                                          CAJA_FILE_MANAGEMENT_PROPERTIES_LABELS_BESIDE_ICONS_WIDGET,
-                                          CAJA_PREFERENCES_ICON_VIEW_LABELS_BESIDE_ICONS);
-    eel_preferences_builder_connect_bool (builder,
-                                          CAJA_FILE_MANAGEMENT_PROPERTIES_ALL_COLUMNS_SAME_WIDTH,
-                                          CAJA_PREFERENCES_COMPACT_VIEW_ALL_COLUMNS_SAME_WIDTH);
-    eel_preferences_builder_connect_bool (builder,
-                                          CAJA_FILE_MANAGEMENT_PROPERTIES_FOLDERS_FIRST_WIDGET,
-                                          CAJA_PREFERENCES_SORT_DIRECTORIES_FIRST);
-    eel_preferences_builder_connect_inverted_bool (builder,
-            CAJA_FILE_MANAGEMENT_PROPERTIES_ALWAYS_USE_BROWSER_WIDGET,
-            CAJA_PREFERENCES_ALWAYS_USE_BROWSER);
+    bind_builder_bool (builder, caja_icon_view_preferences,
+                       CAJA_FILE_MANAGEMENT_PROPERTIES_COMPACT_LAYOUT_WIDGET,
+                       CAJA_PREFERENCES_ICON_VIEW_DEFAULT_USE_TIGHTER_LAYOUT);
+    bind_builder_bool (builder, caja_icon_view_preferences,
+                       CAJA_FILE_MANAGEMENT_PROPERTIES_LABELS_BESIDE_ICONS_WIDGET,
+                       CAJA_PREFERENCES_ICON_VIEW_LABELS_BESIDE_ICONS);
+    bind_builder_bool (builder, caja_compact_view_preferences,
+                       CAJA_FILE_MANAGEMENT_PROPERTIES_ALL_COLUMNS_SAME_WIDTH,
+                       CAJA_PREFERENCES_COMPACT_VIEW_ALL_COLUMNS_SAME_WIDTH);
+    bind_builder_bool (builder, caja_preferences,
+                       CAJA_FILE_MANAGEMENT_PROPERTIES_FOLDERS_FIRST_WIDGET,
+                       CAJA_PREFERENCES_SORT_DIRECTORIES_FIRST);
+    bind_builder_bool_inverted (builder, caja_preferences,
+                                CAJA_FILE_MANAGEMENT_PROPERTIES_ALWAYS_USE_BROWSER_WIDGET,
+                                CAJA_PREFERENCES_ALWAYS_USE_BROWSER);
 
-    eel_preferences_builder_connect_bool (builder,
-                                          CAJA_FILE_MANAGEMENT_PROPERTIES_MEDIA_AUTOMOUNT_OPEN,
-                                          CAJA_PREFERENCES_MEDIA_AUTOMOUNT_OPEN);
-    eel_preferences_builder_connect_bool (builder,
-                                          CAJA_FILE_MANAGEMENT_PROPERTIES_MEDIA_AUTORUN_NEVER,
-                                          CAJA_PREFERENCES_MEDIA_AUTORUN_NEVER);
+    bind_builder_bool (builder, caja_media_preferences,
+                       CAJA_FILE_MANAGEMENT_PROPERTIES_MEDIA_AUTOMOUNT_OPEN,
+                       CAJA_PREFERENCES_MEDIA_AUTOMOUNT_OPEN);
+    bind_builder_bool (builder, caja_media_preferences,
+                       CAJA_FILE_MANAGEMENT_PROPERTIES_MEDIA_AUTORUN_NEVER,
+                       CAJA_PREFERENCES_MEDIA_AUTORUN_NEVER);
 
-    eel_preferences_builder_connect_bool (builder,
-                                          CAJA_FILE_MANAGEMENT_PROPERTIES_TRASH_CONFIRM_WIDGET,
-                                          CAJA_PREFERENCES_CONFIRM_TRASH);
-    eel_preferences_builder_connect_bool (builder,
-                                          CAJA_FILE_MANAGEMENT_PROPERTIES_TRASH_DELETE_WIDGET,
-                                          CAJA_PREFERENCES_ENABLE_DELETE);
-    eel_preferences_builder_connect_bool (builder,
-                                          CAJA_FILE_MANAGEMENT_PROPERTIES_SHOW_HIDDEN_WIDGET,
-                                          CAJA_PREFERENCES_SHOW_HIDDEN_FILES);
-    eel_preferences_builder_connect_bool_slave (builder,
-            CAJA_FILE_MANAGEMENT_PROPERTIES_SHOW_HIDDEN_WIDGET,
-            CAJA_PREFERENCES_SHOW_BACKUP_FILES);
-    eel_preferences_builder_connect_bool (builder,
-                                          CAJA_FILE_MANAGEMENT_PROPERTIES_TREE_VIEW_FOLDERS_WIDGET,
-                                          CAJA_PREFERENCES_TREE_SHOW_ONLY_DIRECTORIES);
+    bind_builder_bool (builder, caja_preferences,
+                       CAJA_FILE_MANAGEMENT_PROPERTIES_TRASH_CONFIRM_WIDGET,
+                       CAJA_PREFERENCES_CONFIRM_TRASH);
 
-    eel_preferences_builder_connect_string_enum_combo_box (builder,
-            CAJA_FILE_MANAGEMENT_PROPERTIES_DEFAULT_VIEW_WIDGET,
-            CAJA_PREFERENCES_DEFAULT_FOLDER_VIEWER,
-            (const char **) default_view_values);
-    eel_preferences_builder_connect_string_enum_combo_box (builder,
-            CAJA_FILE_MANAGEMENT_PROPERTIES_ICON_VIEW_ZOOM_WIDGET,
-            CAJA_PREFERENCES_ICON_VIEW_DEFAULT_ZOOM_LEVEL,
-            (const char **) zoom_values);
-    eel_preferences_builder_connect_string_enum_combo_box (builder,
-            CAJA_FILE_MANAGEMENT_PROPERTIES_COMPACT_VIEW_ZOOM_WIDGET,
-            CAJA_PREFERENCES_COMPACT_VIEW_DEFAULT_ZOOM_LEVEL,
-            (const char **) zoom_values);
-    eel_preferences_builder_connect_string_enum_combo_box (builder,
-            CAJA_FILE_MANAGEMENT_PROPERTIES_LIST_VIEW_ZOOM_WIDGET,
-            CAJA_PREFERENCES_LIST_VIEW_DEFAULT_ZOOM_LEVEL,
-            (const char **) zoom_values);
-    eel_preferences_builder_connect_string_enum_combo_box (builder,
-            CAJA_FILE_MANAGEMENT_PROPERTIES_SORT_ORDER_WIDGET,
-            CAJA_PREFERENCES_ICON_VIEW_DEFAULT_SORT_ORDER,
-            (const char **) sort_order_values);
-    eel_preferences_builder_connect_string_enum_combo_box_slave (builder,
-            CAJA_FILE_MANAGEMENT_PROPERTIES_SORT_ORDER_WIDGET,
-            CAJA_PREFERENCES_LIST_VIEW_DEFAULT_SORT_ORDER);
-    eel_preferences_builder_connect_string_enum_combo_box (builder,
-            CAJA_FILE_MANAGEMENT_PROPERTIES_PREVIEW_TEXT_WIDGET,
-            CAJA_PREFERENCES_SHOW_TEXT_IN_ICONS,
-            (const char **) preview_values);
-    eel_preferences_builder_connect_string_enum_combo_box (builder,
-            CAJA_FILE_MANAGEMENT_PROPERTIES_PREVIEW_IMAGE_WIDGET,
-            CAJA_PREFERENCES_SHOW_IMAGE_FILE_THUMBNAILS,
-            (const char **) preview_values);
-    eel_preferences_builder_connect_string_enum_combo_box (builder,
-            CAJA_FILE_MANAGEMENT_PROPERTIES_PREVIEW_SOUND_WIDGET,
-            CAJA_PREFERENCES_PREVIEW_SOUND,
-            (const char **) preview_values);
-    eel_preferences_builder_connect_string_enum_combo_box (builder,
-            CAJA_FILE_MANAGEMENT_PROPERTIES_PREVIEW_FOLDER_WIDGET,
-            CAJA_PREFERENCES_SHOW_DIRECTORY_ITEM_COUNTS,
-            (const char **) preview_values);
-    eel_preferences_builder_connect_string_enum_combo_box (builder,
-            CAJA_FILE_MANAGEMENT_PROPERTIES_DATE_FORMAT_WIDGET,
-            CAJA_PREFERENCES_DATE_FORMAT,
-            (const char **) date_format_values);
+    bind_builder_bool (builder, caja_preferences,
+                       CAJA_FILE_MANAGEMENT_PROPERTIES_TRASH_DELETE_WIDGET,
+                       CAJA_PREFERENCES_ENABLE_DELETE);
+    bind_builder_bool (builder, caja_preferences,
+                       CAJA_FILE_MANAGEMENT_PROPERTIES_SHOW_HIDDEN_WIDGET,
+                       CAJA_PREFERENCES_SHOW_HIDDEN_FILES);
+    bind_builder_bool (builder, caja_tree_sidebar_preferences,
+                       CAJA_FILE_MANAGEMENT_PROPERTIES_TREE_VIEW_FOLDERS_WIDGET,
+                       CAJA_PREFERENCES_TREE_SHOW_ONLY_DIRECTORIES);
 
-    eel_preferences_builder_connect_string_enum_radio_button (builder,
-            (const char **) click_behavior_components,
-            CAJA_PREFERENCES_CLICK_POLICY,
-            (const char **) click_behavior_values);
-    eel_preferences_builder_connect_string_enum_radio_button (builder,
-            (const char **) executable_text_components,
-            CAJA_PREFERENCES_EXECUTABLE_TEXT_ACTIVATION,
-            (const char **) executable_text_values);
+    bind_builder_enum (builder, caja_preferences,
+                       CAJA_FILE_MANAGEMENT_PROPERTIES_DEFAULT_VIEW_WIDGET,
+                       CAJA_PREFERENCES_DEFAULT_FOLDER_VIEWER,
+                       (const char **) default_view_values);
+    bind_builder_enum (builder, caja_icon_view_preferences,
+                       CAJA_FILE_MANAGEMENT_PROPERTIES_ICON_VIEW_ZOOM_WIDGET,
+                       CAJA_PREFERENCES_ICON_VIEW_DEFAULT_ZOOM_LEVEL,
+                       (const char **) zoom_values);
+    bind_builder_enum (builder, caja_compact_view_preferences,
+                       CAJA_FILE_MANAGEMENT_PROPERTIES_COMPACT_VIEW_ZOOM_WIDGET,
+                       CAJA_PREFERENCES_COMPACT_VIEW_DEFAULT_ZOOM_LEVEL,
+                       (const char **) zoom_values);
+    bind_builder_enum (builder, caja_list_view_preferences,
+                       CAJA_FILE_MANAGEMENT_PROPERTIES_LIST_VIEW_ZOOM_WIDGET,
+                       CAJA_PREFERENCES_LIST_VIEW_DEFAULT_ZOOM_LEVEL,
+                       (const char **) zoom_values);
+    bind_builder_enum (builder, caja_preferences,
+                       CAJA_FILE_MANAGEMENT_PROPERTIES_SORT_ORDER_WIDGET,
+                       CAJA_PREFERENCES_DEFAULT_SORT_ORDER,
+                       (const char **) sort_order_values);
+    bind_builder_enum (builder, caja_preferences,
+                       CAJA_FILE_MANAGEMENT_PROPERTIES_PREVIEW_TEXT_WIDGET,
+                       CAJA_PREFERENCES_SHOW_TEXT_IN_ICONS,
+                       (const char **) preview_values);
+    bind_builder_enum (builder, caja_preferences,
+                       CAJA_FILE_MANAGEMENT_PROPERTIES_PREVIEW_IMAGE_WIDGET,
+                       CAJA_PREFERENCES_SHOW_IMAGE_FILE_THUMBNAILS,
+                       (const char **) preview_values);
+    bind_builder_enum (builder, caja_preferences,
+                       CAJA_FILE_MANAGEMENT_PROPERTIES_PREVIEW_SOUND_WIDGET,
+                       CAJA_PREFERENCES_PREVIEW_SOUND,
+                       (const char **) preview_values);
+    bind_builder_enum (builder, caja_preferences,
+                       CAJA_FILE_MANAGEMENT_PROPERTIES_PREVIEW_FOLDER_WIDGET,
+                       CAJA_PREFERENCES_SHOW_DIRECTORY_ITEM_COUNTS,
+                       (const char **) preview_values);
+    bind_builder_enum (builder, caja_preferences,
+                       CAJA_FILE_MANAGEMENT_PROPERTIES_DATE_FORMAT_WIDGET,
+                       CAJA_PREFERENCES_DATE_FORMAT,
+                       (const char **) date_format_values);
 
-    eel_preferences_builder_connect_uint_enum (builder,
-            CAJA_FILE_MANAGEMENT_PROPERTIES_THUMBNAIL_LIMIT_WIDGET,
-            CAJA_PREFERENCES_IMAGE_FILE_THUMBNAIL_LIMIT,
-            (const guint *) thumbnail_limit_values,
-            G_N_ELEMENTS (thumbnail_limit_values));
+    bind_builder_radio (builder, caja_preferences,
+                        (const char **) click_behavior_components,
+                        CAJA_PREFERENCES_CLICK_POLICY,
+                        (const char **) click_behavior_values);
+    bind_builder_radio (builder, caja_preferences,
+                        (const char **) executable_text_components,
+                        CAJA_PREFERENCES_EXECUTABLE_TEXT_ACTIVATION,
+                        (const char **) executable_text_values);
+
+    bind_builder_uint_enum (builder, caja_preferences,
+                            CAJA_FILE_MANAGEMENT_PROPERTIES_THUMBNAIL_LIMIT_WIDGET,
+                            CAJA_PREFERENCES_IMAGE_FILE_THUMBNAIL_LIMIT,
+                            thumbnail_limit_values,
+                            G_N_ELEMENTS (thumbnail_limit_values));
 
     caja_file_management_properties_dialog_setup_icon_caption_page (builder);
     caja_file_management_properties_dialog_setup_list_column_page (builder);
     caja_file_management_properties_dialog_setup_media_page (builder);
 
-    eel_preferences_add_callback (CAJA_PREFERENCES_MEDIA_AUTORUN_NEVER,
-                                  (EelPreferencesCallback ) caja_file_management_properties_dialog_update_media_sensitivity,
-                                  g_object_ref (builder));
+    g_signal_connect_swapped (caja_media_preferences,
+                              "changed::" CAJA_PREFERENCES_MEDIA_AUTORUN_NEVER,
+                              G_CALLBACK(caja_file_management_properties_dialog_update_media_sensitivity),
+                              builder);
 
 
     /* UI callbacks */
