@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  * Authors: Elliot Lee <sopwith@redhat.com>,
  *          Darin Adler <darin@bentspoon.com>,
@@ -101,7 +101,7 @@ static void eel_gtk_main_quit_all (void)
     g_idle_add (quit_if_in_main_loop, NULL);
 }
 
-static void event_loop_unregister (GtkObject *object)
+static void event_loop_unregister (GtkWidget *object)
 {
     event_loop_registrants = g_slist_remove (event_loop_registrants, object);
     
@@ -111,13 +111,17 @@ static void event_loop_unregister (GtkObject *object)
     }
 }
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+void caja_main_event_loop_register (GtkWidget *object)
+#else
 void caja_main_event_loop_register (GtkObject *object)
+#endif
 {
     g_signal_connect (object, "destroy", G_CALLBACK (event_loop_unregister), NULL);
-    event_loop_registrants = g_slist_prepend (event_loop_registrants, object);
+    event_loop_registrants = g_slist_prepend (event_loop_registrants, GTK_WIDGET (object));
 }
 
-gboolean caja_main_is_event_loop_mainstay (GtkObject *object)
+gboolean caja_main_is_event_loop_mainstay (GtkWidget *object)
 {
     return g_slist_length (event_loop_registrants) == 1
            && event_loop_registrants->data == object;
@@ -148,7 +152,7 @@ void caja_main_event_loop_quit (gboolean explicit)
     }
     while (event_loop_registrants != NULL)
     {
-        gtk_object_destroy (event_loop_registrants->data);
+        gtk_widget_destroy (event_loop_registrants->data);
     }
 }
 
@@ -326,6 +330,8 @@ main (int argc, char *argv[])
     CajaApplication *application;
     GOptionContext *context;
     GFile *file;
+    GFileInfo *fileinfo;
+    GAppInfo *appinfo;
     char *uri;
     char **uris;
     GPtrArray *uris_array;
@@ -494,15 +500,7 @@ main (int argc, char *argv[])
     caja_global_preferences_init ();
 
     /* exit_with_last_window being FALSE, caja can run without window. */
-    exit_with_last_window = eel_preferences_get_boolean (CAJA_PREFERENCES_EXIT_WITH_LAST_WINDOW);
-
-    if (no_desktop)
-    {
-        eel_preferences_set_is_invisible
-        (CAJA_PREFERENCES_SHOW_DESKTOP, TRUE);
-        eel_preferences_set_is_invisible
-        (CAJA_PREFERENCES_DESKTOP_IS_HOME_DIR, TRUE);
-    }
+    exit_with_last_window = g_settings_get_boolean (caja_preferences, CAJA_PREFERENCES_EXIT_WITH_LAST_WINDOW);
 
     application = NULL;
 
@@ -534,13 +532,44 @@ main (int argc, char *argv[])
                 if (file != NULL)
                 {
                     uri = g_file_get_uri (file);
-                    g_object_unref (file);
                     if (uri)
                     {
-                        g_ptr_array_add (uris_array, uri);
+                        fileinfo = g_file_query_info (file, G_FILE_ATTRIBUTE_STANDARD_TYPE, G_FILE_QUERY_INFO_NONE, NULL, NULL);
+                        if (g_file_info_get_file_type(fileinfo) == G_FILE_TYPE_DIRECTORY)
+                        {
+                            g_ptr_array_add (uris_array, uri);
+                        }
+                        else
+                        {
+                            g_object_unref (fileinfo);
+                            fileinfo = g_file_query_info (file, G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE, G_FILE_QUERY_INFO_NONE, NULL, NULL);
+                            appinfo = g_app_info_get_default_for_type (g_file_info_get_content_type (fileinfo), TRUE);
+                            if (appinfo)
+                            {
+                                if (g_strcmp0 (g_app_info_get_executable (appinfo), "caja") != 0)
+                                {
+                                    g_app_info_launch_default_for_uri (uri, NULL, NULL);
+                                }
+                                else
+                                {
+                                    fprintf (stderr, _("caja: set erroneously as default application for '%s' content type.\n"),
+                                             g_file_info_get_content_type (fileinfo));
+                                }
+                                g_object_unref (appinfo);
+                            }
+                            g_free (uri);
+                        }
+                        g_object_unref (fileinfo);
                     }
+                    g_object_unref (file);
                 }
             }
+            if (uris_array->len == 0)
+            {
+                /* Caja is being used only to open files (not directories), so closing */
+                g_strfreev (remaining);
+                return EXIT_SUCCESS;
+            }
             g_ptr_array_add (uris_array, NULL);
             uris = (char**) g_ptr_array_free (uris_array, FALSE);
             g_strfreev (remaining);
