@@ -18,7 +18,7 @@
  *
  *  You should have received a copy of the GNU General Public
  *  License along with this program; if not, write to the Free
- *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  *  Authors: Elliot Lee <sopwith@redhat.com>
  *           John Sullivan <sullivan@eazel.com>
@@ -74,6 +74,8 @@
  */
 #include "caja-desktop-window.h"
 
+#include "glibcompat.h" /* for g_list_free_full */
+
 /* This number controls a maximum character count for a URL that is
  * displayed as part of a dialog. It's fairly arbitrary -- big enough
  * to allow most "normal" URIs to display in full, but small enough to
@@ -83,10 +85,13 @@
 
 static void begin_location_change                     (CajaWindowSlot         *slot,
         GFile                      *location,
+        GFile                      *previous_location,
         GList                      *new_selection,
-        CajaLocationChangeType  type,
+        CajaLocationChangeType      type,
         guint                       distance,
-        const char                 *scroll_pos);
+        const char                 *scroll_pos,
+        CajaWindowGoToCallback      callback,
+        gpointer                    user_data);
 static void free_location_change                      (CajaWindowSlot         *slot);
 static void end_location_change                       (CajaWindowSlot         *slot);
 static void cancel_location_change                    (CajaWindowSlot         *slot);
@@ -122,13 +127,10 @@ caja_window_report_selection_changed (CajaWindowInfo *window)
 static void
 set_displayed_location (CajaWindowSlot *slot, GFile *location)
 {
-    CajaWindow *window;
     GFile *bookmark_location;
     gboolean recreate;
     char *name;
 
-    window = slot->pane->window;
-
     if (slot->current_location_bookmark == NULL || location == NULL)
     {
         recreate = TRUE;
@@ -511,18 +513,22 @@ caja_window_slot_open_location_full (CajaWindowSlot *slot,
                                      GFile *location,
                                      CajaWindowOpenMode mode,
                                      CajaWindowOpenFlags flags,
-                                     GList *new_selection)
+                                     GList *new_selection,
+                                     CajaWindowGoToCallback callback,
+                                     gpointer user_data)
 {
     CajaWindow *window;
     CajaWindow *target_window;
     CajaWindowPane *pane;
     CajaWindowSlot *target_slot;
     CajaWindowOpenFlags slot_flags;
-    gboolean do_load_location = TRUE;
+    gboolean existing = FALSE;
     GFile *old_location;
     char *old_uri, *new_uri;
     int new_slot_position;
     GList *l;
+    gboolean target_navigation, target_same;
+    gboolean is_desktop;
 
     window = slot->pane->window;
 
@@ -546,87 +552,78 @@ caja_window_slot_open_location_full (CajaWindowSlot *slot,
     g_assert (!((flags & CAJA_WINDOW_OPEN_FLAG_NEW_WINDOW) != 0 &&
                 (flags & CAJA_WINDOW_OPEN_FLAG_NEW_TAB) != 0));
 
+    is_desktop = CAJA_IS_DESKTOP_WINDOW (window);
+    target_same = is_desktop &&
+    		!caja_desktop_window_loaded (CAJA_DESKTOP_WINDOW (window));
 
     old_location = caja_window_slot_get_location (slot);
+
     switch (mode)
     {
     case CAJA_WINDOW_OPEN_ACCORDING_TO_MODE :
-        if (eel_preferences_get_boolean (CAJA_PREFERENCES_ALWAYS_USE_BROWSER))
-        {
-            target_window = window;
-            if (CAJA_IS_SPATIAL_WINDOW (window))
-            {
-                if (!CAJA_SPATIAL_WINDOW (window)->affect_spatial_window_on_next_location_change)
-                {
-                    target_window = caja_application_create_navigation_window
-                                    (window->application,
-                                     NULL,
-                                     gtk_window_get_screen (GTK_WINDOW (window)));
-                }
-                else
-                {
-                    CAJA_SPATIAL_WINDOW (window)->affect_spatial_window_on_next_location_change = FALSE;
-                }
-            }
-            else if ((flags & CAJA_WINDOW_OPEN_FLAG_NEW_WINDOW) != 0)
-            {
-                target_window = caja_application_create_navigation_window
-                                (window->application,
-                                 NULL,
-                                 gtk_window_get_screen (GTK_WINDOW (window)));
+        if (g_settings_get_boolean (caja_preferences, CAJA_PREFERENCES_ALWAYS_USE_BROWSER)) {
+            /* always use browser: if we're on the desktop the target is a new navigation window,
+            * otherwise it's the same window.
+            */
+            if (is_desktop) {
+            	target_navigation = TRUE;
+            } else {
+            	target_same = TRUE;
             }
+        } else if (flags & CAJA_WINDOW_OPEN_FLAG_NEW_WINDOW) {
+            /* if it's specified to open a new window, and we're not using spatial,
+             * the target is a navigation.
+             */
+            target_navigation = TRUE;
         }
-        else if (CAJA_IS_SPATIAL_WINDOW (window))
-        {
-            if (!CAJA_SPATIAL_WINDOW (window)->affect_spatial_window_on_next_location_change)
-            {
-                target_window = caja_application_present_spatial_window_with_selection (
-                                    window->application,
-                                    window,
-                                    NULL,
-                                    location,
-                                    new_selection,
-                                    gtk_window_get_screen (GTK_WINDOW (window)));
-                do_load_location = FALSE;
-            }
-            else
-            {
-                CAJA_SPATIAL_WINDOW (window)->affect_spatial_window_on_next_location_change = FALSE;
-                target_window = window;
-            }
-        }
-        else if (flags & CAJA_WINDOW_OPEN_FLAG_NEW_WINDOW)
-        {
-            target_window = caja_application_create_navigation_window
-                            (window->application,
-                             NULL,
-                             gtk_window_get_screen (GTK_WINDOW (window)));
-        }
-        else
-        {
-            target_window = window;
-        }
-        break;
-    case CAJA_WINDOW_OPEN_IN_SPATIAL :
-        target_window = caja_application_present_spatial_window (
-                            window->application,
-                            window,
-                            NULL,
-                            location,
-                            gtk_window_get_screen (GTK_WINDOW (window)));
         break;
     case CAJA_WINDOW_OPEN_IN_NAVIGATION :
-        target_window = caja_application_create_navigation_window
-                        (window->application,
-                         NULL,
-                         gtk_window_get_screen (GTK_WINDOW (window)));
+        target_navigation = TRUE;
         break;
     default :
-        g_warning ("Unknown open location mode");
+        g_critical ("Unknown open location mode");
         g_object_unref (old_location);
         return;
     }
 
+    /* now get/create the window according to the mode */
+    if (target_same) {
+        target_window = window;
+    } else if (target_navigation) {
+        target_window = caja_application_create_navigation_window
+            (window->application,
+             NULL,
+             gtk_window_get_screen (GTK_WINDOW (window)));
+    } else {
+        target_window = caja_application_get_spatial_window
+            (window->application,
+             window,
+             NULL,
+             location,
+             gtk_window_get_screen (GTK_WINDOW (window)),
+             &existing);
+    }
+
+    /* if the spatial window is already showing, present it and set the
+     * new selection, if present.
+     */
+    if (existing) {
+        target_slot = target_window->details->active_pane->active_slot;
+
+        gtk_window_present (GTK_WINDOW (target_window));
+
+        if (new_selection != NULL && slot->content_view != NULL) {
+            caja_view_set_selection (target_slot->content_view, new_selection);
+        }
+
+        /* call the callback successfully */
+        if (callback != NULL) {
+            callback (window, NULL, user_data);
+        }
+
+        return;
+    }
+
     g_assert (target_window != NULL);
 
     if ((flags & CAJA_WINDOW_OPEN_FLAG_NEW_TAB) != 0 &&
@@ -636,7 +633,7 @@ caja_window_slot_open_location_full (CajaWindowSlot *slot,
 
         slot_flags = 0;
 
-        new_slot_position = eel_preferences_get_enum (CAJA_PREFERENCES_NEW_TAB_POSITION);
+        new_slot_position = g_settings_get_enum (caja_preferences, CAJA_PREFERENCES_NEW_TAB_POSITION);
         if (new_slot_position == CAJA_NEW_TAB_POSITION_END)
         {
             slot_flags = CAJA_WINDOW_OPEN_SLOT_APPEND;
@@ -676,24 +673,19 @@ caja_window_slot_open_location_full (CajaWindowSlot *slot,
         }
     }
 
-    if ((!do_load_location) ||
-            (target_window == window && target_slot == slot &&
-             old_location && g_file_equal (old_location, location)))
-    {
-        if (old_location)
-        {
-            g_object_unref (old_location);
+    if ((target_window == window && target_slot == slot &&
+             old_location && g_file_equal (old_location, location))) {
+
+        if (callback != NULL) {
+            callback (window, NULL, user_data);
         }
+
+	g_object_unref (old_location);
         return;
     }
 
-    if (old_location)
-    {
-        g_object_unref (old_location);
-    }
-
-    begin_location_change (target_slot, location, new_selection,
-                           CAJA_LOCATION_CHANGE_STANDARD, 0, NULL);
+    begin_location_change (target_slot, location, old_location, new_selection,
+                           CAJA_LOCATION_CHANGE_STANDARD, 0, NULL, callback, user_data);
 
     /* Additionally, load this in all slots that have no location, this means
        we load both panes in e.g. a newly opened dual pane window. */
@@ -701,12 +693,16 @@ caja_window_slot_open_location_full (CajaWindowSlot *slot,
     {
         pane = l->data;
         slot = pane->active_slot;
-        if (slot->location == NULL && slot->pending_location == NULL)
-        {
-            begin_location_change (slot, location, new_selection,
-                                   CAJA_LOCATION_CHANGE_STANDARD, 0, NULL);
+        if (slot->location == NULL && slot->pending_location == NULL) {
+            begin_location_change (slot, location, old_location, new_selection,
+                                   CAJA_LOCATION_CHANGE_STANDARD, 0, NULL, NULL, NULL);
         }
     }
+
+    if (old_location)
+    {
+        g_object_unref (old_location);
+    }
 }
 
 void
@@ -724,7 +720,7 @@ caja_window_slot_open_location (CajaWindowSlot *slot,
 
     caja_window_slot_open_location_full (slot, location,
                                          CAJA_WINDOW_OPEN_ACCORDING_TO_MODE,
-                                         flags, NULL);
+                                         flags, NULL, NULL, NULL);
 }
 
 void
@@ -742,7 +738,7 @@ caja_window_slot_open_location_with_selection (CajaWindowSlot *slot,
     }
     caja_window_slot_open_location_full (slot, location,
                                          CAJA_WINDOW_OPEN_ACCORDING_TO_MODE,
-                                         flags, selection);
+                                         flags, selection, NULL, NULL);
 }
 
 
@@ -766,7 +762,7 @@ caja_window_slot_go_home (CajaWindowSlot *slot, gboolean new_tab)
     home = g_file_new_for_path (g_get_home_dir ());
     caja_window_slot_open_location_full (slot, home,
                                          CAJA_WINDOW_OPEN_ACCORDING_TO_MODE,
-                                         flags, NULL);
+                                         flags, NULL, NULL, NULL);
     g_object_unref (home);
 }
 
@@ -855,9 +851,23 @@ caja_window_slot_content_view_matches_iid (CajaWindowSlot *slot,
     {
         return FALSE;
     }
-    return eel_strcmp (caja_view_get_view_id (slot->content_view), iid) == 0;
+    return g_strcmp0 (caja_view_get_view_id (slot->content_view), iid) == 0;
 }
 
+static gboolean
+report_callback (CajaWindowSlot *slot,
+                 GError *error)
+{
+    if (slot->open_callback != NULL) {
+        slot->open_callback (slot->pane->window, error, slot->open_callback_user_data);
+        slot->open_callback = NULL;
+        slot->open_callback_user_data = NULL;
+
+        return TRUE;
+    }
+
+    return FALSE;
+}
 
 /*
  * begin_location_change
@@ -865,11 +875,14 @@ caja_window_slot_content_view_matches_iid (CajaWindowSlot *slot,
  * Change a window's location.
  * @window: The CajaWindow whose location should be changed.
  * @location: A url specifying the location to load
+ * @previous_location: The url that was previously shown in the window that initialized the change, if any
  * @new_selection: The initial selection to present after loading the location
  * @type: Which type of location change is this? Standard, back, forward, or reload?
  * @distance: If type is back or forward, the index into the back or forward chain. If
  * type is standard or reload, this is ignored, and must be 0.
  * @scroll_pos: The file to scroll to when the location is loaded.
+ * @callback: function to be called when the location is changed.
+ * @user_data: data for @callback.
  *
  * This is the core function for changing the location of a window. Every change to the
  * location begins here.
@@ -877,16 +890,21 @@ caja_window_slot_content_view_matches_iid (CajaWindowSlot *slot,
 static void
 begin_location_change (CajaWindowSlot *slot,
                        GFile *location,
+                       GFile *previous_location,
                        GList *new_selection,
                        CajaLocationChangeType type,
                        guint distance,
-                       const char *scroll_pos)
+                       const char *scroll_pos,
+                       CajaWindowGoToCallback callback,
+                       gpointer user_data)
 {
     CajaWindow *window;
     CajaDirectory *directory;
     CajaFile *file;
     gboolean force_reload;
     char *current_pos;
+    GFile *from_folder;
+    GFile *parent;
 
     g_assert (slot != NULL);
     g_assert (location != NULL);
@@ -894,6 +912,25 @@ begin_location_change (CajaWindowSlot *slot,
               || type == CAJA_LOCATION_CHANGE_FORWARD
               || distance == 0);
 
+    /* If there is no new selection and the new location is
+     * a (grand)parent of the old location then we automatically
+     * select the folder the previous location was in */
+    if (new_selection == NULL && previous_location != NULL &&
+        g_file_has_prefix (previous_location, location)) {
+        from_folder = g_object_ref (previous_location);
+        parent = g_file_get_parent (from_folder);
+        while (parent != NULL && !g_file_equal (parent, location)) {
+            g_object_unref (from_folder);
+            from_folder = parent;
+            parent = g_file_get_parent (from_folder);
+        }
+        if (parent != NULL) {
+            new_selection = g_list_prepend (NULL, g_object_ref(from_folder));
+        }
+        g_object_unref (from_folder);
+        g_object_unref (parent);
+    }
+
     window = slot->pane->window;
     g_assert (CAJA_IS_WINDOW (window));
     g_object_ref (window);
@@ -914,6 +951,9 @@ begin_location_change (CajaWindowSlot *slot,
 
     slot->pending_scroll_to = g_strdup (scroll_pos);
 
+    slot->open_callback = callback;
+    slot->open_callback_user_data = user_data;
+
     directory = caja_directory_get (location);
 
     /* The code to force a reload is here because if we do it
@@ -1093,14 +1133,12 @@ mount_not_mounted_callback (GObject *source_object,
                             gpointer user_data)
 {
     MountNotMountedData *data;
-    CajaWindow *window;
     CajaWindowSlot *slot;
     GError *error;
     GCancellable *cancellable;
 
     data = user_data;
     slot = data->slot;
-    window = slot->pane->window;
     cancellable = data->cancellable;
     g_free (data);
 
@@ -1245,11 +1283,15 @@ got_file_info_for_view_selection_callback (CajaFile *file,
         }
         create_content_view (slot, view_id);
         g_free (view_id);
+
+        report_callback (slot, NULL);
     }
     else
     {
-        display_view_selection_failure (window, file,
-                                        location, error);
+    	if (!report_callback (slot, error)) {
+        	display_view_selection_failure (window, file,
+                	                        location, error);
+        }
 
         if (!gtk_widget_get_visible (GTK_WIDGET (window)))
         {
@@ -1261,11 +1303,6 @@ got_file_info_for_view_selection_callback (CajaFile *file,
             {
                 g_assert (caja_application_get_n_windows () == 1);
 
-                /* Make sure we re-use this window */
-                if (CAJA_IS_SPATIAL_WINDOW (window))
-                {
-                    CAJA_SPATIAL_WINDOW (window)->affect_spatial_window_on_next_location_change = TRUE;
-                }
                 /* the user could have typed in a home directory that doesn't exist,
                    in which case going home would cause an infinite loop, so we
                    better test for that */
@@ -1288,13 +1325,13 @@ got_file_info_for_view_selection_callback (CajaFile *file,
                 }
                 else
                 {
-                    gtk_object_destroy (GTK_OBJECT (window));
+                    gtk_widget_destroy (GTK_WIDGET (window));
                 }
             }
             else
             {
                 /* Since this is a window, destroying it will also unref it. */
-                gtk_object_destroy (GTK_OBJECT (window));
+                gtk_widget_destroy (GTK_WIDGET (window));
             }
         }
         else
@@ -1365,7 +1402,7 @@ create_content_view (CajaWindowSlot *slot,
     }
 
     if (slot->content_view != NULL &&
-            eel_strcmp (caja_view_get_view_id (slot->content_view),
+            g_strcmp0 (caja_view_get_view_id (slot->content_view),
                         view_id) == 0)
     {
         /* reuse existing content view */
@@ -1396,7 +1433,7 @@ create_content_view (CajaWindowSlot *slot,
                            FALSE,
                            TRUE);
 
-        eel_g_object_list_free (slot->pending_selection);
+    	g_list_free_full (slot->pending_selection, g_object_unref);
         slot->pending_selection = NULL;
     }
     else if (slot->location != NULL)
@@ -1407,7 +1444,7 @@ create_content_view (CajaWindowSlot *slot,
                            selection,
                            FALSE,
                            TRUE);
-        eel_g_object_list_free (selection);
+    	g_list_free_full (selection, g_object_unref);
     }
     else
     {
@@ -1465,7 +1502,7 @@ load_new_location (CajaWindowSlot *slot,
         caja_view_set_selection (view, selection_copy);
     }
 
-    eel_g_object_list_free (selection_copy);
+    g_list_free_full (selection_copy, g_object_unref);
 }
 
 /* A view started to load the location its viewing, either due to
@@ -1706,15 +1743,12 @@ found_mount_cb (GObject *source_object,
 {
     FindMountData *data = user_data;
     GMount *mount;
-    CajaWindowSlot *slot;
 
     if (g_cancellable_is_cancelled (data->cancellable))
     {
         goto out;
     }
 
-    slot = data->slot;
-
     mount = g_file_find_enclosing_mount_finish (G_FILE (source_object),
             res,
             NULL);
@@ -1922,7 +1956,7 @@ free_location_change (CajaWindowSlot *slot)
     }
     slot->pending_location = NULL;
 
-    eel_g_object_list_free (slot->pending_selection);
+    g_list_free_full (slot->pending_selection, g_object_unref);
     slot->pending_selection = NULL;
 
     /* Don't free pending_scroll_to, since thats needed until
@@ -1976,7 +2010,7 @@ cancel_location_change (CajaWindowSlot *slot)
                            selection,
                            TRUE,
                            FALSE);
-        eel_g_object_list_free (selection);
+    	g_list_free_full (selection, g_object_unref);
     }
 
     end_location_change (slot);
@@ -2033,8 +2067,8 @@ caja_window_report_view_failed (CajaWindow *window,
     if (fallback_load_location != NULL)
     {
         /* We loose the pending selection change here, but who cares... */
-        begin_location_change (slot, fallback_load_location, NULL,
-                               CAJA_LOCATION_CHANGE_FALLBACK, 0, NULL);
+        begin_location_change (slot, fallback_load_location, NULL, NULL,
+                               CAJA_LOCATION_CHANGE_FALLBACK, 0, NULL, NULL, NULL);
         g_object_unref (fallback_load_location);
     }
 
@@ -2053,7 +2087,6 @@ display_view_selection_failure (CajaWindow *window, CajaFile *file,
     char *error_message;
     char *detail_message;
     char *scheme_string;
-    GtkDialog *dialog;
 
     /* Some sort of failure occurred. How 'bout we tell the user? */
     full_uri_for_display = g_file_get_parse_name (location);
@@ -2152,7 +2185,7 @@ display_view_selection_failure (CajaWindow *window, CajaFile *file,
         detail_message = g_strdup_printf (_("Error: %s\nPlease select another viewer and try again."), error->message);
     }
 
-    dialog = eel_show_error_dialog (error_message, detail_message, NULL);
+    eel_show_error_dialog (error_message, detail_message, NULL);
 
     g_free (uri_for_display);
     g_free (error_message);
@@ -2247,6 +2280,7 @@ caja_navigation_window_back_or_forward (CajaNavigationWindow *window,
     CajaNavigationWindowSlot *navigation_slot;
     GList *list;
     GFile *location;
+    GFile *old_location;
     guint len;
     CajaBookmark *bookmark;
 
@@ -2273,19 +2307,25 @@ caja_navigation_window_back_or_forward (CajaNavigationWindow *window,
         caja_window_slot_open_location_full (slot, location,
                                              CAJA_WINDOW_OPEN_ACCORDING_TO_MODE,
                                              CAJA_WINDOW_OPEN_FLAG_NEW_TAB,
-                                             NULL);
+                                             NULL, NULL, NULL);
     }
     else
     {
         char *scroll_pos;
 
+        old_location = caja_window_slot_get_location (slot);
         scroll_pos = caja_bookmark_get_scroll_pos (bookmark);
         begin_location_change
         (slot,
-         location, NULL,
+         location, old_location, NULL,
          back ? CAJA_LOCATION_CHANGE_BACK : CAJA_LOCATION_CHANGE_FORWARD,
          distance,
-         scroll_pos);
+         scroll_pos,
+         NULL, NULL);
+
+        if (old_location) {
+            g_object_unref (old_location);
+        }
 
         g_free (scroll_pos);
     }
@@ -2320,11 +2360,12 @@ caja_window_slot_reload (CajaWindowSlot *slot)
         selection = caja_view_get_selection (slot->content_view);
     }
     begin_location_change
-    (slot, location, selection,
-     CAJA_LOCATION_CHANGE_RELOAD, 0, current_pos);
+    (slot, location, location, selection,
+     CAJA_LOCATION_CHANGE_RELOAD, 0, current_pos,
+     NULL, NULL);
     g_free (current_pos);
     g_object_unref (location);
-    eel_g_object_list_free (selection);
+    g_list_free_full (selection, g_object_unref);
 }
 
 void
